/***************************************************************************
 *   Copyright (C) 2006 by George Kiagiadakis   *
 *   gkiagia@users.sourceforge.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QStringList>
#include <QString>
#include <QMessageBox>
#include <QApplication>
#include <QDir>
#include <QFile>
#include <iostream>
#include "winelauncher.h"
#include "confighandler.h"
#include "macros.h"

WineLauncher::WineLauncher()
	:InteractiveProcess()
{
	envIsSet = FALSE;
	setSilent(FALSE);
	logIsOn = FALSE;
	setQuitMode(WineLauncher::continueRunning);
}

WineLauncher::~WineLauncher(){}

void WineLauncher::startWine(QString winApp, QStringList arguments)
{
	arguments.prepend(winApp);
	start("wine", arguments, QIODevice::ReadWrite);
	connect(this, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(exitCodeHandler(int, QProcess::ExitStatus)) );
}

void WineLauncher::startTerminal(QString cfgName, QString winApp, QStringList arguments, QString logfile, QString debugStr, QString workdir)
{
	QProcess *w = new QProcess;
	arguments.prepend(winApp);
	arguments.prepend(cfgName);
	arguments.prepend("--launch");
	arguments.prepend("--ask");
	
	if ( !workdir.isEmpty() )
	{
		arguments.prepend(workdir);
		arguments.prepend("--workdir");
	}

	if (!logfile.isEmpty())
	{
		arguments.prepend(logfile);
		arguments.prepend("--log");
	}

	arguments.prepend(QApplication::arguments()[0]); // the name of the calling process ("qtwine" or "QtWine.bin")
	arguments.prepend("-e");
	
	if ( !debugStr.isEmpty() )
		w->setEnvironment( systemEnvironment() << QString("WINEDEBUG=%1").arg(debugStr) );


	w->start(QtWine::settings->value("apps/terminal", "konsole").toString() , arguments);
	if (!w->waitForStarted())
	{
		QMessageBox::critical(0,"Error", "Could not open terminal. Check options in manager and try again.", "OK");
	}
}

void WineLauncher::setWorkDir(QString dir)
{
	if ( !dir.isEmpty() )
		setWorkingDirectory(dir);
	//else the workdir of the calling process is used
}

void WineLauncher::setCfgName(QString name)
{
	QString configPath = cfgHandler->getCfgPath(name);
	if ( configPath.isEmpty() )
	{
		qDebug("Error: %s: no such configuration", qPrintable(name));
		exit(1); // probably qtwine is running in text mode and the configuration doesn't exist so we have to exit.
	}

	if ( envIsSet )
		setEnvironment(environment() << QString("WINEPREFIX=%1").arg(configPath) );
	else
	{
		setEnvironment(systemEnvironment() << QString("WINEPREFIX=%1").arg(configPath) );
		envIsSet = TRUE;
	}
}

void WineLauncher::setDebug(QString str)
{
	if ( envIsSet )
		setEnvironment(environment() << QString("WINEDEBUG=%1").arg(str) );
	else
	{
		setEnvironment(systemEnvironment() << QString("WINEDEBUG=%1").arg(str) );
		envIsSet = TRUE;
	}
}

void WineLauncher::setSilent(bool silentOn)
{
	silent = silentOn;
	if (silent)
		setReadChannelMode(QProcess::SeparateChannels);
	else
		setReadChannelMode(QProcess::ForwardedChannels);
}

void WineLauncher::setQuitMode(WineLauncher::qAppQuitType choice)
{
	qAppQuitAfterExit = choice;
}

void WineLauncher::exitCodeHandler( int exitCode, QProcess::ExitStatus exitStatus )
{

	if (exitStatus == QProcess::NormalExit)
	{
		QByteArray txt;
		txt += "\n\n*****************************************\n\n";
		txt += "Wine exited. Exit code "; txt += QString("%1").arg(exitCode);
		txt += "\n\n*****************************************\n\n";
		if (silent == FALSE)
		{
			QFile out;
			out.open(stdout, QIODevice::WriteOnly);
			out.write( txt );
			out.close();
		}
		if (logIsOn)
		{
			log.write( txt );
		}
	}
	else if (exitStatus == QProcess::CrashExit)
	{
		QByteArray txt;
		txt += "\n\n*****************************************\n\n";
		txt += "Wine CRASHED !!!";
		txt += "\n\n*****************************************\n\n";
		if (silent == FALSE)
		{
			out.open(stdout, QIODevice::WriteOnly);
			out.write( txt );
			out.close();
		}
		if (logIsOn)
		{
			log.write( txt );
		}
	}
	
	if (logIsOn)
		log.close();

	if ( qAppQuitAfterExit == WineLauncher::ask )
	{
		std::cout << "Press enter to exit\n\n";
		getchar();
		exit(0);
	}
	else if ( qAppQuitAfterExit == WineLauncher::quit )
		exit(0);
	else
		delete this;
}

void WineLauncher::setLogFile(QString filename)
{
	if (filename.isEmpty()) return;

	log.setFileName(filename);
	if (!log.open(QIODevice::WriteOnly | QIODevice::Text))
	{
		qDebug("\n\nWARNING: Could not open log file !!!! Check permissions.\n\n");
		return;
	}

	logIsOn = TRUE;
	setReadChannelMode(QProcess::SeparateChannels); // switch to that mode to be able to catch messages in stdout/stderr
	connect(this , SIGNAL(readyReadStandardOutput()), this, SLOT(readStandardOutput()) );
	connect(this , SIGNAL(readyReadStandardError()), this, SLOT(readStandardError()) );
}

void WineLauncher::readStandardOutput()
{
	QByteArray ba = readAllStandardOutput();
	log.write( ba );
	if (silent == FALSE)
	{
		out.open(stdout, QIODevice::WriteOnly);
		out.write( ba );
		out.close();
	}
}

void WineLauncher::readStandardError()
{
	QByteArray ba = readAllStandardError();
	log.write( ba );
	if (silent == FALSE)
	{
		err.open(stderr, QIODevice::WriteOnly);
		err.write( ba );
		err.close();
	}
}


